//A C++ OCR Sample
//******************************************************************************
//                        Nicomsoft OCR DLL interface
//                    Copyright (c) 2010-2013 Nicomsoft
//                 Copyright (c) 2010-2013 Michael Kapustin
//                           www.nsocr.com
//******************************************************************************

#include <cstdlib>
#include "stdio.h"
#include "string.h"
#include "malloc.h"
#include "locale.h"
#include "wchar.h"
#include "NSOCR.h"

using namespace std;

#ifndef __x86_64__
#define OCR_BIN_LIB   L"../../Bin/libNSOCR.so"
#else
#define OCR_BIN_LIB   L"../../Bin_64/libNSOCR.so"
#endif

int main(int argc, char** argv) 
{
  UNICODECHAR fname[512];	
  int CfgObj, OcrObj, ImgObj, SvrObj, res, n;
  wchar_t* txt;
  
  setlocale(LC_ALL, "");  
  
  // Create OCR engine wrapper class
  TNSOCR ocr(OCR_BIN_LIB);

  UNICODECHAR libver[16];
  ocr.Engine_GetVersion(libver);
  fwprintf (stdout, L"Nicomsoft OCR for Linux (v%ls) - C++ Sample\n", libver); 
    
  if (argc < 2)
  {
	  fwprintf (stdout, L"Syntax:\nocrsmp picture_filename <out_pdf_filename>\n"); 
	  exit(1);
  }
    
  //initialize OCR engine, create objects and load configuration
  res = ocr.Engine_InitializeAdvanced(&CfgObj, &OcrObj, &ImgObj); 
  if (res > ERROR_FIRST) 
  {
	  fwprintf (stdout, L"Error initialize library: 0x%x", res); 
    exit(1);
  };
   
  //by default this option is disabled because it takes about 10% of total recognition time
  //enable it to demonstrate this feature
  ocr.Cfg_SetOption(CfgObj, BT_DEFAULT, L"Zoning/FindBarcodes", L"1");
  
  //also enable auto-detection of image inversion
  ocr.Cfg_SetOption(CfgObj, BT_DEFAULT, L"ImgAlizer/Inversion", L"2");
  
  
  //load some image for OCR
  mbsrtowcs(fname, (const char **)&argv[1], sizeof(fname)/sizeof(UNICODECHAR),  NULL);	
  res = ocr.Img_LoadFile(ImgObj, fname);   
  if (res > ERROR_FIRST) 
  {
  	  fwprintf (stdout, L"Error open file\n"); 
	  exit(1);
  };
  
  res = ocr.Img_OCR(ImgObj, OCRSTEP_FIRST, OCRSTEP_LAST, OCRFLAG_NONE); //perform OCR
  if (res > ERROR_FIRST) 
  {
	  fwprintf (stdout, L"Error library: 0x%x", res); 	    
	  exit(1);
  };
  
  // Save to PDF 
  if (argc > 2)
  {
	  mbsrtowcs(fname, (const char **)&argv[2], sizeof(fname)/sizeof(UNICODECHAR),  NULL);	
	  
	  ocr.Svr_Create(CfgObj, SVR_FORMAT_PDF, &SvrObj); //create Saver object, output format is PDF
	  res = ocr.Svr_NewDocument(SvrObj); //start new document
  
	  res = ocr.Svr_AddPage(SvrObj, ImgObj, FMT_EXACTCOPY); //add OCRed image page to output document  
	  if (res > ERROR_FIRST) 
	  {
		  fwprintf (stdout, L"Error add page: 0x%x", res); 	    
		  exit(1);
	  }; //insert error handler here
	  
	  res = ocr.Svr_SaveToFile(SvrObj, fname); //save OCRed image to PDF file    
	  if (res > ERROR_FIRST) 
	  {
		  fwprintf (stdout, L"Error create .pdf file: 0x%x", res); 	    
		  exit(1);
	  }; //insert error handler here
	  
	  fwprintf (stdout, L"File: %ls successfully created!", fname); 	    

  }
  else
  {
	  n = ocr.Img_GetImgText(ImgObj, NULL, 0, FMT_EXACTCOPY) + 1; //get buffer size plus terminating NULL character
      txt = (wchar_t*) malloc(sizeof(UNICODECHAR) * n); //allocate memory for text
      ocr.Img_GetImgText(ImgObj, txt, n, FMT_EXACTCOPY); //get text
  
      fwprintf (stdout, L"Recognized text:\n %ls\n", txt);
  
      //use "txt" variable now
      free(txt); //free memory 
  }
 
  return 0;
}

